# ==========================================================================
# DoubleLinkedList.py: Double linked lists are linked lists with pointers to
# both the next (following) and previous nodes.
# 
# Key features: 
# 
# 1. Although worst case time complexities of all operations in a doubly
#    linked list are same as that of a singly linked list, some operations
#    are faster, e.g., lookup (or searching) is O(n/2) because seaching can
#    begin at both ends. 
# 
# ==========================================================================

# Create generic node to store data ...

class Node():
    def __init__(self, data):
        self.data = data
        self.next = None
        self.previous = None

# Create generic node to store data ...

class DoubleLinkedList():
    def __init__(self):
        self.head = None
        self.tail = self.head
        self.length = 0

    # Print list ...

    def print_list(self):
        if self.head == None:
            print("--- Empty")
        else:
            current_node = self.head
            i = 0
            while current_node != None:
                print("--- Item %d: %s ..." % (i, current_node.data))
                current_node = current_node.next
                i = i + 1

    # Append element to list ...

    def append(self, data):
        new_node = Node(data)
        if self.head == None: 
            self.head = new_node
            self.tail = self.head
            self.length += 1
            return
        else:
            new_node.previous = self.tail
            self.tail.next = new_node
            self.tail      = new_node 
            self.length += 1
            return

    # Prepend node ...

    def prepend(self, data):
        new_node = Node(data)
        if self.head == None:
            self.head = new_node
            self.tail = self.head
            self.length += 1
            return
        else:
            new_node.next      = self.head 
            self.head.previous = new_node 
            self.head = new_node
            self.length += 1
            return

    # Insert new node ...

    def insert(self, position, data):
        if position == 0:
            self.prepend(data) 
            return
        if position >= self.length:
            self.append(data) 
            return
        else:
            new_node = Node(data)
            current_node = self.head
            for i in range(position - 1): 
                current_node = current_node.next
            new_node.previous = current_node 
            new_node.next     = current_node.next 
            current_node.next = new_node 
            new_node.next.previous = new_node 
            self.length += 1
            return

    # Delete node by value ...

    def delete_by_value(self, data):
        if self.head == None:
            print("--- Linked List is empty. Nothing to delete.")
            return

        current_node = self.head
        if current_node.data == data:
            self.head = self.head.next
            if self.head == None or self.head.next==None: 
                self.tail = self.head
            if self.head != None:
                self.head.previous = None 
            self.length -= 1
            return
        try:  
            while current_node!= None and current_node.next.data != data:
                current_node = current_node.next
            if current_node!=None:
                current_node.next = current_node.next.next
                if current_node.next != None: 
                    current_node.next.previous = current_node 
                else:
                    self.tail = current_node 
                self.length -= 1
                return
        except AttributeError:
            print("Given value not found.")
            return

    # Delete by position ...

    def delete_by_position(self, position):
        if self.head == None:
            print("--- Linked List is empty. Nothing to delete.")
            return

        if position == 0:
            self.head = self.head.next
            #print(self.head)
            if self.head == None or self.head.next == None:
                self.tail = self.head
            if self.head != None:
                self.head.previous = None 
            self.length -= 1
            return

        if position>=self.length:
            position = self.length-1

        current_node = self.head
        for i in range(position - 1):
            current_node = current_node.next
        current_node.next = current_node.next.next
        if current_node.next != None:
            current_node.next.previous = current_node
        else:
            self.tail = current_node
        self.length -= 1
        return
