# ==================================================================
# Person.py: Simple model of a Person. The scope of variables
# _age, _ssn, and _dob are protected to Person and all subclasses. 
# 
# Written by: Mark Austin                              November 2022 
# ==================================================================

from datetime import date

class Person:
   _age = 0   # <-- age ...
   _ssn = 0   # <-- social security number ...
   _dob = 0   # <-- date of birth ...

   # ----------------------------
   # Constructor method ...
   # ----------------------------

   def __init__(self, fname, lname, dob ):
     self._firstname = fname
     self._lastname  = lname
     self._dob       = dob
     self._age       = self.calculateAge()

   # ----------------------------
   # Get first and last names ...
   # ----------------------------

   def getFirstName(self): 
     return self._firstname 

   def getLastName(self): 
     return self._lastname 

   # ----------------------------
   # Set/get date of birth ...
   # ----------------------------

   def setDob(self, dob):
     self._dob = dob

   def getDob(self, dob):
     return self._dob

   # ----------------------------
   # Calculate age ...
   # ----------------------------

   def calculateAge(self):
      today = date.today()
      age   = today.year - self._dob.year - ((today.month, today.day) < (self._dob.month, self._dob.day))
      return age

   # ----------------------------
   # Set/get/print age ...
   # ----------------------------

   def setAge(self, age):
     self._age = age

   def getAge(self):
     return self._age

   # ----------------------------------------
   # Set/get/print social security number ...
   # ----------------------------------------

   def setSSN(self, ssn ):
     self._ssn = ssn

   def getSSN(self):
     return self._ssn

   # ----------------------------------------
   # return string represention of object ...
   # ----------------------------------------

   def __str__(self):
       return "Person: {:6.2f} {:6.2f}: age = {:f} ".format( self._firstname, self._lastname, self._age )
