# ===============================================================
# TestDataTransformation01.py: Exercise data transformations in
# Pandas.
# 
# Basic operation include:
# 
# --
# --
# --
# --
#                                                  November, 2022 
# ===============================================================

import pandas as pd
import numpy as np

# main method ...

def main():
    print("--- Enter TestDataTransformation01.main()                    ... ");
    print("--- ======================================================== ... ");

    # Manually create and print tiny dataframe ...

    print("--- ");
    print("--- Part 01: Manually create and print small dataset ... ");
    print("--- ");

    mydataset = {
       'cars': [ "BMW", "Honda", "Acura"], 
       'year': [ 2013,     2017,    2022]
    }

    myvar = pd.DataFrame(mydataset)
    print(myvar)

    # Create dataframe from 1-d np array

    print("--- ");
    print("--- Part 02: Create dataframe from 1-d np array ... ");
    print("--- ");

    myvar = pd.DataFrame( np.arange(1,8) )
    print(myvar)

    # Create dataframe from 2-d np array

    print("--- ");
    print("--- Part 03: Create dataframe from 2-d np array ... ");
    print("--- ");

    df = pd.DataFrame( [ [1,2], [3,4], [5,6] ] )
    print(df)

    print("--- ");
    print("--- dataframe info and shape ... ");
    print("--- ");

    print(df.info() )
    print(df.shape)

    # Load CSV file into a Pandas dataframe ...

    print("--- ");
    print("--- Part 04: load CSV file into Pandas dataframe ... ");
    print("--- ");
 
    df = pd.read_csv('../data/AirPassengers.csv')
    print(df.head()) 

    print("--- ");
    print("--- dataframe info and shape ... ");
    print("--- ");

    print(df.info() )
    print(df.shape)

    # Create simple dataframe from multiple series ...

    print("--- ");
    print("--- Part 05.1: Create simple dataframe from mutiple series ... ");
    print("--- ");

    data = {
       "calories": [520, 480, 400],
       "duration": [ 50,  48,  40]
    }

    myvar = pd.DataFrame(data)
    print(myvar)

    print("--- ");
    print("--- Part 05.2: Give each row a name ... ");
    print("--- ");

    myvar = pd.DataFrame(data, index = ["day1", "day2", "day3"])
    print(myvar) 

    # Create JSON object (same format as Python dictionary) ...

    print("--- ");
    print("--- Part 06: Create JSON object (same format as Python dictionary) ... ");
    print("--- ");

    data = {
      "Duration":{ "0":60, "1":60, "2":60, "3":45, "4":45, "5":60 },
      "Pulse":{ "0":110, "1":117, "2":103, "3":109, "4":117, "5":102 },
      "Maxpulse":{ "0":130, "1":145, "2":135, "3":175, "4":148, "5":127 },
      "Calories":{ "0":409, "1":479, "2":340, "3":282, "4":406, "5":300 }
    }

    df = pd.DataFrame(data)
    print(df) 

    print("--- ");
    print("--- Part 07: Selecting columns of a dataframe ... ");
    print("--- ");

    print( df[ [ 'Duration','Calories'] ].head() )

    print("--- ");
    print("--- Part 08: Selecting rows of a dataframe ... ");
    print("--- ");

    print("--- row 1 ... ");
    print( df.loc['1'].head() )
    print("--- row 2 ... ");
    print( df.loc['2'].head() )
    print("--- row 3 ... ");
    print( df.loc['3'].head() )

    print("--- ");
    print("--- Part 09: Evaluate rows via boolean selection ... ");
    print("--- ");

    print("--- query 1: df.Duration == 60 ... ");
    print( df.Duration == 60 )

    print("--- ");
    print("--- Part 10: Select rows via boolean selection ... ");
    print("--- ");

    print("--- query 2: df.Duration == 60 ... ");
    query2 = df[( df.Duration == 60 )] ['Duration']
    print( query2 )

    print("--- query 3: df.Calories > 300 ... ");
    query3 = df[( df.Calories > 300 )] ['Calories']
    print( query3 )

    print("--- ");
    print("--- Part 11: Transform dataFrame columns to numpy array ...");
    print("--- ");

    df = pd.DataFrame(data) 
    print (df)

    col01 = df['Duration'].values
    print("--- Duration column --> numpy array: %s ..." %( col01 ))
    col02 = df['Pulse'].values
    print("--- Pulse column --> numpy array: %s ..." %( col02 ))

    print("--- ======================================================== ... ");
    print("--- Leave TestDataTransformation01.main()                    ... ");

# call the main method ...

main()
